// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TARGET.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:24  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Globalization;
using Stochastix.Utilities;

namespace Stochastix.Business.Calendars
{
	/// <summary>
	/// TARGET calendar.
	/// </summary>
	/// <remarks>
	/// Holidays:
	/// <list type="table">
	/// <item><description>Saturdays</description></item>
	/// <item><description>Sundays</description></item>
	/// <item><description>New Year's Day, January 1st</description></item>
	/// <item><description>Good Friday (since 2000)</description></item>
	/// <item><description>Easter Monday (since 2000)</description></item>
	/// <item><description>Labour Day, May 1st (since 2000)</description></item>
	/// <item><description>Christmas, December 25th</description></item>
	/// <item><description>Day of Goodwill, December 26th (since 2000)</description></item>
	/// <item><description>New Year's Eve, December 31st (1998, 1999 and 2001 only)</description></item>
	/// </list>
	/// </remarks>	
	/// <seealso href="http://www.national-holidays.com/">National Holidays (website)</seealso>
	[ RcsId("$Id: TARGET.cs,v 1.1.1.1 2003/02/01 18:25:24 jens Exp $") ] 
	[ ComVisible(true) ]
	[ ClassInterface(ClassInterfaceType.None) ]
	[ FactoryItem("TARGET"), FactoryItem("Euro"), FactoryItem("EUR") ]
	public sealed class TARGET : WesternCalendarBase, IBusinessCalendar
	{
		/// <summary>
		/// Parameterless COM constructor - use <see cref="Instance"/> member instead.
		/// </summary>
		/// <remarks>
		/// This constructor is provided for COM compatibility only.
		/// Use the static member <see cref="Instance"/> to get an object
		/// of this type.
		/// </remarks>
		[ Obsolete("Use TARGET.Instance in .NET applications.") ]
		public TARGET() 
			: base( "TARGET", CultureInfo.InvariantCulture)
		{}

		#region FactoryItem pattern

		[ Obsolete() ] // just to ignore the CS0618 warning below
		static TARGET()
		{
			Instance = new TARGET();	// CS0618
		}

		/// <summary>
		/// A static instance of this type.
		/// </summary>
		static public readonly IBusinessCalendar Instance;

		#endregion

		/// <summary>
		/// Implementation of the public <see cref="IsBusinessDay"/> method.
		/// </summary>
		/// <remarks>
		/// This method must be implemented by concrete
		/// calendar implementations.
		/// </remarks>
		/// <param name="d">The day value, between 1 and 31.</param>
		/// <param name="m">The month, one of <see cref="Month"/>.</param>
		/// <param name="y">The year, between 1 and 9999.</param>
		/// <param name="w">A <see cref="DayOfWeek"/> enumerated constant that
		/// indicates the day of the week. 
		/// This property value ranges from zero, indicating Sunday,
		/// to six, indicating Saturday.</param>
		/// <param name="dd">The day of the year, between 1 and 366.</param>
		/// <param name="em">The day of Easter Monady in the year, between 1 and 366.</param>
		/// <returns><c>True</c> when the given day is a business day.</returns>
		override protected bool IsBusinessDay(
			int d, Month m, int y,
			DayOfWeek w, int dd, int em ) 
		{
			if ((w == DayOfWeek.Saturday || w == DayOfWeek.Sunday)
				// New Year's Day
				|| (d == 1 && m == Month.January)
				// Good Friday
				|| (dd == em-3 && y >= 2000)
				// Easter Monday
				|| (dd == em && y >= 2000)
				// Labour Day
				|| (d == 1 && m == Month.May && y >= 2000)
				// Christmas
				|| (d == 25 && m == Month.December)
				// Day of Goodwill
				|| (d == 26 && m == Month.December && y >= 2000)
				// December 31st, 1998 and 1999 only
				|| (d == 31 && m == Month.December && (y == 1998 || y == 1999 || y == 2001)))
				return false;
			return true;
		}

	}
}
