// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ActualActualAFB.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:25  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.Business.DayCounters
{
	/// <summary>
	/// European Actual/Actual day count convention.
	/// </summary>
	/// <seealso href="http://www.isda.org/c_and_a/pdf/mktc1198.pdf">
	/// ISDA: EMU and market conventions
	/// </seealso>
	[ RcsId("$Id: ActualActualAFB.cs,v 1.1.1.1 2003/02/01 18:25:25 jens Exp $") ]
	[ ComVisible(true), ClassInterface(ClassInterfaceType.None) ]
	[ FactoryItem("act/act (Euro)"), FactoryItem("act/act(e)"), FactoryItem("actacte") ]
	public sealed class ActualActualAFB : DayCounterBase, IDayCounter 
	{
		/// <summary>
		/// Parameterless COM constructor - use <see cref="Instance"/> member instead.
		/// </summary>
		/// <remarks>
		/// This constructor is provided for COM compatibility only.
		/// Use the static member <see cref="Instance"/> to get an object
		/// of this type.
		/// </remarks>
		[ Obsolete("Use the static Instance member in .NET applications.") ]
		public ActualActualAFB() : this("act/act (Euro)") 
		{}

		internal ActualActualAFB(String name) : base(name) 
		{}

		#region FactoryItem pattern

		[ Obsolete() ] // just to ignore the CS0618 warning below
		static ActualActualAFB()
		{
			Instance = new ActualActualAFB();	// CS0618
		}

		/// <summary>
		/// A static instance of this type.
		/// </summary>
		static public readonly ActualActualAFB Instance;
		
		#endregion

		override protected double YearFractionImpl(
			DateTime startDate, DateTime endDate, 
			DateTime refPeriodStart, DateTime refPeriodEnd) 
		{
			DateTime newendDate=endDate, temp=endDate;
			double sum = 0.0;
			while (temp > startDate) 
			{
				temp = newendDate.AddYears(-1);
				if (temp.Day==28 && temp.Month==2
					&& DateTime.IsLeapYear(temp.Year)) 
				{
					temp.AddDays(1);
				}
				if (temp>=startDate) 
				{
					sum += 1.0;
					newendDate = temp;
				}
			}

			double den = 365.0;

			if (DateTime.IsLeapYear(newendDate.Year)) 
			{
				temp = new DateTime(newendDate.Year, 2, 29 );
				if (newendDate>temp && startDate<=temp)
					den += 1.0;
			} 
			else if (DateTime.IsLeapYear(startDate.Year)) 
			{
				temp = new DateTime(startDate.Year, 2, 29 );
				if (newendDate>temp && startDate<=temp)
					den += 1.0;
			}

			return sum+ActualDays(startDate, newendDate) / den;
		}

	}

}
