// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Functional.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:10  jens
 *  Initial import
 *
 *  Revision 1.3  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.2  2002/10/15 12:43:33  stochastix
 *  updated license
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;

namespace Stochastix.Numerics
{
	/// <summary>
	/// Helper to apply a unary function to a list of values.
	/// </summary>
	/// <remarks>
	/// The unary function can either implement 
	/// <see cref="IUnaryFunction"/> or is a delegate
	/// of type <see cref="UnaryFunction"/>.
	/// </remarks>
	/// <seealso cref="IUnaryFunction"/>
	/// <seealso cref="UnaryFunction">UnaryFunction delegate</seealso>
	[ ComVisible(false) ]
	public sealed class Functional
	{
		private Functional()
		{}

		/// <summary>
		/// Apply the given unary function to a list of values.
		/// </summary>
		/// <param name="f">
		/// The unary function implementing <see cref="IUnaryFunction"/>.
		/// </param>
		/// <param name="x">
		/// The given function is applied to all values in the given
		/// <see cref="IList"/>.
		/// </param>
		public static void Apply(IUnaryFunction f, IList x) 
		{
			for( int i=0; i< x.Count; i++)
				x[i] = f.Value((double)x[i]);
		}

		/// <summary>
		/// Apply the given unary function to a list of values.
		/// </summary>
		/// <param name="f">
		/// The <see cref="UnaryFunction"/> delegate applied to all
		/// values of x.
		/// </param>
		/// <param name="x">
		/// The given function is applied to all values in the given
		/// <see cref="IList"/>.
		/// </param>
		public static void Apply(UnaryFunction f, IList x) 
		{
			for( int i=0; i< x.Count; i++)
				x[i] = f((double)x[i]);
		}
	}
}
