// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestSolvers1D.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:13  jens
 *  Initial import
 *
 *  Revision 1.3  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.2  2002/10/15 12:45:28  stochastix
 *  updated license
 *
 *  Revision 1.1  2002/08/28 15:43:36  stochastix
 *  moved into dotNum module
 *
 *  Revision 1.1.1.1  2002/08/15 11:45:37  stochastix
 *  Initial revision
 *
 *  Revision 1.1  2002/08/07 19:15:21  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.Numerics.Tests
{
	using System;
	using System.Diagnostics;
	using NUnit.Framework;
	using Stochastix.Numerics.Solvers1D;

	/// <summary>
	/// 
	/// </summary>
	public class TestSolvers1D: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestSolvers1D(String name): base(name) {}

		// declare private members here used for test setup

		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
		}

		private object[] solvers = { 
			new Brent(), 
			new Bisection(), 
			new FalsePosition(),
			new Ridder(), 
			new Secant(),
			new Newton(),
			new NewtonSafe()
		};

		private double[] accuracies = new double[] { 1e-4, 1e-6, 1e-8 };

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
			solvers = null;
		}

		private class TestFunc : ObjectiveFunction
		{
			override public double Value(double x) 
			{
				return x*x-1.0;
			}
			override public double Derivative(double x) 
			{
				return 2.0 * x;
			}

		}

		public void TestSolvers()
		{
			IObjectiveFunction objFun = new TestFunc();
			foreach ( ISolver1D solver in solvers) 
			{
				foreach ( double accuracy in accuracies)
				{
					string testMsg = String.Format( "{0}, accuracy={1:e1}", 
						solver, accuracy);
					double root = solver.Solve(objFun, accuracy, 1.5, 0.1);
					Assertion.Assert( testMsg, 
						Math.Abs(root-1.0) <= accuracy);
					root = solver./*Bracketed*/Solve(
						objFun,accuracy,1.5,0.0,1.0);
					Assertion.Assert( testMsg + " (bracketed)", 
						Math.Abs(root-1.0) <= accuracy);
				}
			}
		}

	}
}
