// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestVector.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:13  jens
 *  Initial import
 *
 *  Revision 1.3  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.2  2002/10/15 12:54:00  stochastix
 *  updated license
 *
 *  Revision 1.1  2002/08/28 15:43:36  stochastix
 *  moved into dotNum module
 *
 *  Revision 1.3  2002/08/20 22:32:38  stochastix
 *  TestIndices() is now in TestBLAS.cs
 *
 *  Revision 1.2  2002/08/20 16:44:55  stochastix
 *  updated
 *
 *  Revision 1.1.1.1  2002/08/15 11:45:37  stochastix
 *  Initial revision
 *
 *  Revision 1.1  2002/08/09 18:10:09  stochastix
 *  Preparing for preview release
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.Numerics.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Org.Mentalis.Utilities;
	using Stochastix.Numerics;

	/// <summary>
	/// 
	/// </summary>
	public class TestVector: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestVector(String name): base(name) {}

		// declare private members here used for test setup

		double[] indexValues;

		private const double tolerance = 1e-8;
		
		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
			indexValues = new double[]
				{ 0.5, -100.0, 0.001, 1.0, -0.5 };
		}

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
		}

		public void DontTestVmlPerformance()
		{
			for(int d=5; d<250; d+=25)
			{
				StopWatch s = new StopWatch();
				Vector comp = new Vector(d, 1.0, 0.01);
				comp.Sin(); comp.Asin();
				Vector test;

				GC.Collect();
				test = comp.Clone();
				for( int i=0; i< 10000; i++ )
				{
					if( i==3) s.Reset(); // three warmup rounds
					test.Sin(); test.Asin();
				}
				double avg = s.Peek()/10.0;
				for( int j=0; j<test.Count; j++)
					Assert( Math.Abs( test[j] - comp[j]) <= 1e-10);

				Console.WriteLine("dim={0}, avg/100ms={1}", d, avg);
			}
		}

		public void TestSqrt()
		{
			 Vector v = new Vector(40, 0.0, 0.1);
			// Vector sq = Vector.Sqrt(v);
			// printVector(sq);
			Vector inv = Vector.Inv(v);
			printVector(inv);

			Vector nul = new Vector(40, 0.0);
			Vector one = new Vector(40, 1.0);
			Vector pi = new Vector(40, Math.PI);
			Vector nan = new Vector(40, 1.0/0.0);
			Vector pinf = new Vector(40, Double.PositiveInfinity);
			Vector ninf = new Vector(40, Double.NegativeInfinity);
			Vector expt = new Vector(40, Math.Log(Double.MaxValue) +1.0);

			printVector(Vector.Cos(pinf));
			printVector(Vector.Cos(nan));
		}
			

		private double printVector(Vector v)
		{
			// TODO: Vector.ToString() (see Matrix.IFormatable)

			double sum = 0.0;
			Console.Write("{ ");
			foreach (double d in v)
			{
				sum += d;
				Console.Write("{0:f6} ", d);
			}
			Console.WriteLine(" }");
			return sum;
		}

	}
}
