// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: IFloatingRateCoupon.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:27  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.QuantLib.Indexes;
using Stochastix.Business;

namespace Stochastix.QuantLib.CashFlows
{

	/// <summary>
	/// Coupon at par on a term structure.
	/// </summary>
	/// <remarks>
	/// This interface is mainly for COM interop.
	/// Since COM interop doesn't support the typelib export
	/// of inherited interfaces, we have to repeat all the
	/// inherited members here!
	/// </remarks>
	[
	ComVisible(true),
	Description("Coupon at par on a term structure."),
	InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual)
	]
	public interface IFloatingRateCoupon : ICoupon
	{ 

#region Repeated declaration of ICoupon

		/// <summary>
		/// A stringified representation of the cash flow.
		/// </summary>
		/// <remarks>
		/// This method is used for output and comparison between indexes.
		///	It is <b>not</b> meant to be used for writing switch-on-type code.
		/// </remarks>
		/// <returns>A String representing the object.</returns>
		[ Description("A stringified representation of this object.") ]
		[ DispId(0) ] // DISPID_VALUE
		new String ToString();

		/// <summary>
		/// The undiscounted amount of the cash flow.
		/// </summary>
		/// <remarks>
		/// The amount is not discounted, i.e., it is the actual amount
		/// paid at the cash flow date.
		/// </remarks>
		[ Description("The undiscounted amount of the cash flow.") ]
		new double Amount
		{
			get;
		}

		/// <summary>
		/// The date at which the cash flow is settled.
		/// </summary>
		/// <remarks>
		/// The amount is not discounted, i.e., it is the actual amount
		/// paid at the cash flow date.
		/// </remarks>
		[ Description("The date at which the cash flow is settled.") ]
		new DateTime Date
		{
			get;
		}

		/// <summary>
		/// Nominal value.
		/// </summary>
		[ Description("Nominal value.") ]
		new double Nominal
		{
			get;
		}

		/// <summary>
		/// Start of the accrual period.
		/// </summary>
		[ Description("Start of the accrual period.") ]
		new DateTime AccrualStartDate
		{
			get;
		}

		/// <summary>
		/// End of the accrual period.
		/// </summary>
		[ Description("End of the accrual period.") ]
		new DateTime AccrualEndDate
		{
			get;
		}

		/// <summary>
		/// Accrual period as fraction of year.
		/// </summary>
		[ Description("Accrual period as fraction of year.") ]
		new double AccrualPeriod
		{
			get;
		}

		/// <summary>
		/// Accrual period in days.
		/// </summary>
		[ Description("Accrual period in days.") ]
		new int AccrualDays
		{
			get;
		}

		/// <summary>
		/// Day counter for accrual calculation.
		/// </summary>
		[ Description("Day counter for accrual calculation.") ]
		new IDayCounter DayCounter
		{
			get;
		}

		/// <summary>
		/// Accrued amount at the given date.
		/// </summary>
		/// <param name="date"></param>
		/// <returns></returns>
		[ Description("Accrued amount at the given date.") ]
		new double AccruedAmount(DateTime date);

#endregion

		/// <summary>
		/// Index.
		/// </summary>
		[ Description("Index.") ]
		IXibor Index 
		{
			get;
		}

		/// <summary>
		/// Fixing days.
		/// </summary>
		[ Description("Fixing days.") ]
		int FixingDays
		{
			get;
		}

		/// <summary>
		/// Fixing.
		/// </summary>
		[ Description("Fixing.") ]
		double Fixing
		{
			get;
		}

		/// <summary>
		/// Spread.
		/// </summary>
		[ Description("Spread.") ]
		double Spread
		{
			get;
		}

	}

}
