// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: DZero.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:28  jens
 *  Initial import
 *
 *  Revision 1.2  2002/08/30 18:13:01  stochastix
 *  small changes for DotNum and license
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.QuantLib.FiniteDifferences
{
	/// <summary>
	/// D<sub>0</sub> matricial representation.
	/// </summary>
	/// <remarks>
	/// The differential operator D<sub>0</sub> discretizes the
	/// first derivative with the second-order formula
	/// \f[ \frac{\partial u_{i}}{\partial x} \approx
	///		\frac{u_{i+1}-u_{i-1}}{2h} = D_{0} u_{i} 
	///	\f]
	/// </remarks>
	[ RcsId("$Id: DZero.cs,v 1.1.1.1 2003/02/01 19:01:28 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("D_{0} matricial representation.") ]
	public class DZero : TridiagonalOperator, IOperator
	{
		public DZero(int gridPoints, double h) : base(gridPoints) 
		{
			SetFirstRow(-1/h, 1/h);				// linear extrapolation
			SetMidRows(-1/(2*h), 0.0, 1/(2*h));
			SetLastRow(-1/h, 1/h);				// linear extrapolation
		}
	}
}
