// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Lattice2D.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:30  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:41  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Lattices
{
	/// <summary>
	/// Two-dimensional lattice.
	/// </summary>
	/// <remarks>
	/// This lattice is based on two trinomial trees and primarly
	/// used for the G2 short-rate model.
	/// </remarks>
	[ RcsId("$Id: Lattice2D.cs,v 1.1.1.1 2003/02/01 19:01:30 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class Lattice2D : Lattice 
	{
		public Lattice2D(TrinomialTree tree1, TrinomialTree tree2, 
			double correlation) : base(tree1.TimeGrid, 9)
		{
			this.tree1 = tree1;
			this.tree2 = tree2;
			this.rho   = Math.Abs(correlation);

			if (correlation < 0.0) 
			{
				m = new double[,]
					{
						{ -1.0, -4.0, 5.0 },
						{ -4.0, 8.0, -4.0 },
						{ 5.0, -4.0, -1.0 }
					};
			} 
			else 
			{
				m = new double[,]
					{
						{ 5.0, -4.0, -1.0 },
						{ -4.0, 8.0, -4.0 },
						{ -1.0, -4.0, 5.0 }
					};
			}
		}

		protected Tree tree1, tree2;
		private double[,] m;
		private double rho;

		override public int Count(int i)  
		{ 
			return tree1.Count(i)*tree2.Count(i); 
		}

		override protected int Descendant(int i, int index, int branch)
		{
            int modulo = tree1.Count(i);
            int index1 = index % modulo;
            int index2 = index / modulo;
            int branch1 = branch % 3;
            int branch2 = branch / 3;

            modulo = tree1.Count(i+1);
            return tree1.Descendant(i, index1, branch1) +
                   tree2.Descendant(i, index2, branch2) * modulo;
        }

		override protected double Probability(int i, int index, int branch)
		{
			int modulo = tree1.Count(i);
			int index1 = index % modulo;
			int index2 = index / modulo;
			int branch1 = branch % 3;
			int branch2 = branch / 3;
			
            double prob1 = tree1.Probability(i, index1, branch1);
            double prob2 = tree2.Probability(i, index2, branch2);
            return prob1*prob2 + rho * m[branch1,branch2] / 36.0;
        }

	}
}
