// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: PricingEngine.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:32  jens
 *  Initial import
 *
 *  Revision 1.2  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Data;
using System.Diagnostics;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.PricingEngines
{
	/// <summary>
	/// Abstract base class for data-driven pricing engines.
	/// </summary>
	[ RcsId("$Id: PricingEngine.cs,v 1.1.1.1 2003/02/01 19:01:32 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("Abstract base class for data-driven pricing engines.") ]
	public abstract class PricingEngine : IPricingEngine
	{

		#region Implementation of IPricingEngine
		public DataSet NewDataSet()
		{
			DataSet ds = new DataSet(this.UniqueId);
			DataTable arguments = NewArgumentsTable();
			DataTable results = NewResultsTable();
			ds.Tables.Add( arguments );
			return ds;
		}
		public void Calculate(DataSet dataSet)
		{
			DataTable arguments = dataSet.Tables["Arguments"];

			// get or make the results table
			DataTable results;
			if( dataSet.Tables.Contains("Results") )
			{
				results = dataSet.Tables["Results"];
			}
			else
			{
				results = NewResultsTable();
				dataSet.Tables.Add( results );
				dataSet.Relations.Add( arguments.PrimaryKey, results.PrimaryKey);
			}

//			// get the relation between the two tables
//			DataRelation relArgsResults = null;
//			foreach( DataRelation rel in ds.Relations)
//				if(rel.ParentTable == arguments && rel.ChildTable==results)
//					relArgsResults = rel;
//			if( relArgsResults == null )
//				throw new ApplicationException("Could not find a relation between " + 
//					"arguments and results.");

			// calculate all rows
			object[] keys = results.PrimaryKey;
			foreach( DataRow args in arguments.Rows ) 
			{
				DataRow result = results.NewRow();
				Calculate(args, result);
				try
				{
					result.SetParentRow(args);
				} 
				catch
				{}
//				DataRow prev = results.Rows.Find(keyValues);
//				prev.
//				if(Contains(keyValues));
				try
				{
					results.Rows.Add( result );
				} 
				catch( ConstraintException /* e */) 
				{
					// Debug.WriteLine(e);
					object[] keyValues = new object[keys.Length];
					for(int i=0; i<keys.Length; i++)
						keyValues[i] = result[(DataColumn)keys[i]];
					results.Rows.Find(keyValues).Delete();
					results.Rows.Add(result);
				}
			}
		}

		abstract public void Calculate(DataRow args, DataRow results);

		public void Validate(DataSet dataSet)
		{
			DataTable arguments = dataSet.Tables["Arguments"];
			// DataTable results   = dataSet.Tables["Results"];
			foreach( DataRow row in arguments.Rows )
				this.Validate(row);
		}
		abstract public void Validate(DataRow dataRow);
		abstract public string UniqueId
		{
			get;
		}
		#endregion

		static private DataTable argumentsTable;
		protected virtual DataTable NewArgumentsTable()
		{
			if( argumentsTable == null ) 
			{
				argumentsTable = new DataTable("Arguments");
				DataColumn argumentsId = new DataColumn("id", typeof(long));
				argumentsId.AutoIncrement = true;
				argumentsTable.Columns.Add( argumentsId );
				// argumentsId made NotDBNull and Unique below!
				argumentsTable.PrimaryKey = new DataColumn[]{ argumentsId };
			}
			return argumentsTable.Clone();
		}

		static private DataTable resultsTable;
		protected virtual DataTable NewResultsTable()
		{
			if( resultsTable == null ) 
			{
				resultsTable = new DataTable("Results");
				DataColumn resultsId = new DataColumn("id", typeof(long));
				resultsId.AutoIncrement = true;
				resultsTable.Columns.Add( resultsId );
				// resultsId made NotDBNull and Unique below!
				resultsTable.PrimaryKey = new DataColumn[]{ resultsId };
			}
			return resultsTable.Clone();
		}
	}
}
