// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestOperators.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *  Revision 1.3  2002/08/30 18:23:25  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/15 18:15:28  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:15:20  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.QuantLib.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using QuantLib;
	using Stochastix.QuantLib.FiniteDifferences;
	using Stochastix.Numerics;
	using Stochastix.Numerics.Distributions;


	/// <summary>
	/// 
	/// </summary>
	public class TestOperators: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestOperators(String name): base(name) {}

		// declare private members here used for test setup

		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
			x = new Vector(N, xMin, h);
			y = x.Apply( new UnaryFunction(normal.Value));
			yDerivative = x.Apply( new UnaryFunction(normal.Derivative));
			yIntegrated = x.Apply( cum );
		}

		private const double average = 0.0;
		private const double sigma = 1.0;
		private const double xMin = average - 4*sigma;
		private const double xMax = average + 4*sigma;
		private const int N = 10001;
		private const double h = (xMax-xMin)/(N-1);
		private Vector x, y, yIntegrated, yDerivative;

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
			x = y = yIntegrated = yDerivative = null;
		}

		private double norm(Vector f, double h)
		{
			double sum = 0.0;
			foreach (double d in f) 
				sum += d*d;
			// numeric integral of f^2
			double res = h* ( sum - 0.5*f[0]*f[0] - 
				0.5*f[f.Count-1]*f[f.Count-1] );
			return Math.Sqrt(res);
		}

		private NormalDistribution normal = 
			new NormalDistribution(average, sigma);
		private CumulativeNormalDistribution cum = 
			new CumulativeNormalDistribution(average, sigma);

		public void TestDZero()
		{
			IOperator D = new DZero(N,h);
			Vector yTemp = D.ApplyTo(yIntegrated);
			yTemp.Subtract(y);
			Assertion.Assert( norm(yTemp,h) <= 1.0e-6);
		}

		public void TestDPlusDMinus()
		{
			IOperator D = new DPlusDMinus(N,h);
			Vector yTemp = D.ApplyTo(yIntegrated);
			yTemp.Subtract(yDerivative);
			Assertion.Assert( norm(yTemp,h) <= 1.0e-4);
		}
	}
}
