// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestMarketElements.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *  Revision 1.3  2002/08/30 18:23:25  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/15 18:15:28  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:15:20  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.QuantLib.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Stochastix.QuantLib.MarketElements;
	using Stochastix.Numerics;


	/// <summary>
	/// 
	/// </summary>
	public class TestMarketElements: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestMarketElements(String name): base(name) {}

		// declare private members here used for test setup

		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
			quote = new SimpleMarketElement(0.0);
			dep   =	new DerivedMarketElement( quote, 
				new UnaryFunction( spread));
			comp = new CompositeMarketElement(quote, dep,
				new BinaryFunction( add ));
			counter = 0;
		}

		private SimpleMarketElement		quote;
		private DerivedMarketElement	dep;
		private CompositeMarketElement	comp;

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
			quote = null;
			dep = null;
			comp = null;
		}

		/// <summary>
		/// Unary test function.
		/// </summary>
		static double spread(double x) { return x*1.1; }

		/// <summary>
		/// Binary test function.
		/// </summary>
		static double add(double x, double y) { return x+y; }


		/// <summary>
		/// Event handler.
		/// </summary>
		protected void update(object sender, EventArgs e) 
		{ 
			counter++;
		}
		private int counter;

		public void TestObservability()
		{
			Assert(counter==0);
			quote.Changed += new EventHandler(update);

			// testing that events fire
			quote.Value += 0.1;
			Assert(counter==1);
			quote.Value += 0.1;
			Assert(counter==2);

			// testing unregister
			quote.Changed -= new EventHandler(update);
			quote.Value += 0.1;
			Assert(counter==2);

			// Testing derived market element
			Assert(spread(quote.Value) == dep.Value);

			// testing that events get passed along
			dep.Changed += new EventHandler(update);
			quote.Value += 0.1;
			Assert(counter==3);
			Assert(spread(quote.Value) == dep.Value);
			Assert( comp.Value == quote.Value + dep.Value);

			// testing that changed functions trigger events
			dep.Function = new UnaryFunction(spread);
			Assert(counter==4);
			Assert(spread(quote.Value) == dep.Value);
			Assert( comp.Value == quote.Value + dep.Value);

			// we will now see 3 events at once:
			// quote -> dep -> us
			//          dep -> comp -> us
			// quote -> comp -> us
			comp.Changed += new EventHandler(update);
			quote.Value += 0.1;
			Assert( counter == 7 );
			Assert( comp.Value == quote.Value + dep.Value );

			// unregister and check again
			comp.Changed -= new EventHandler(update);
			dep.Changed -= new EventHandler(update);
			quote.Value += 0.1;
			Assert(counter==7);
			Assert(spread(quote.Value) == dep.Value);
		}

	}
}

