// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestPiecewiseFlatForward.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *  Revision 1.2  2002/08/30 18:23:25  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.1  2002/08/07 19:15:21  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.QuantLib.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Stochastix.Business;
	using Stochastix.Business.Calendars;
	using Stochastix.Business.DayCounters;
	using Stochastix.QuantLib.TermStructures;
	using Stochastix.QuantLib.MarketElements;
	using Stochastix.QuantLib.Indexes;
	using Stochastix.QuantLib.Instruments;


	/// <summary>
	/// 
	/// </summary>
	public class TestPiecewiseFlatForward: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestPiecewiseFlatForward(String name): base(name) {}

		// declare private members here used for test setup

		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
		}

		// private ITermStructure termStructure;

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
		}

		private const double TOLERANCE = 1.0e-9;

		private object[,] depositData = new object[,]
		{
			{ "1w", 4.559 }, { "1m", 4.581 }, { "2m", 4.573 },
			{ "3m", 4.557 } , { "6m", 4.496 }, { "9m", 4.490 }
		};
		private object[,] swapData = new object[,]
		{
			{ "1y", 4.54}, { "2y", 4.63}, { "3y", 4.75},
			{ "4y", 4.86}, { "5y", 4.99}, { "6y", 5.11},
			{ "7y", 5.23}, { "8y", 5.33}, { "9y", 5.41},
			{ "10y", 5.47}, { "12y", 5.60}, { "15y", 5.75},
			{ "20y", 5.89}, { "25y", 5.95}, { "30y", 5.96}
		};

		public void TestPFF()
		{
			IBusinessCalendar calendar = TARGET.Instance;
			// TODO: running this test on a holiday fails!!!!
			DateTime todaysDate = calendar.Roll(DateTime.Today);
			int settlementDays = 2;
			DateTime settlementDate = calendar.Advance(
				todaysDate, settlementDays, TimeUnit.Days,
				RollingConvention.Following);
			int fixingDays = 2;
			ArrayList instruments = new ArrayList();
        
			// deposits
			RollingConvention rollingConvention = 
				RollingConvention.ModifiedFollowing;
			IDayCounter dayCounter = Actual360.Instance;
			for( int i=0; i<depositData.GetLength(0); i++)
			{
				Period tenor = Period.Parse((string)depositData[i,0]);
				double rate = (double)depositData[i,1] / 100.0;
				instruments.Add( new DepositRateHelper(
					rate, tenor.Length, tenor.Units, settlementDays, 
					calendar, rollingConvention, dayCounter));
			}

			// swaps
			RollingConvention swapRollingConvention = 
				RollingConvention.ModifiedFollowing;
			int fixedFrequency = 1;
			bool fixedIsAdjusted = false;
			IDayCounter fixedDayCount = Thirty360US.Instance;
			int floatingFrequency = 2;
			for( int i=0; i<swapData.GetLength(0); i++)
			{
				Period tenor = Period.Parse((string)swapData[i,0]);
				double rate = (double)swapData[i,1] / 100.0;
				instruments.Add( new SwapRateHelper(rate,
					tenor.Length, tenor.Units, settlementDays, 
					calendar, swapRollingConvention, 
					fixedFrequency, fixedIsAdjusted, fixedDayCount,
					floatingFrequency));
			}

			// instantiate curve
			ITermStructure termStructure = 
				new PiecewiseFlatForward( todaysDate, settlementDate,
				instruments, Actual360.Instance, 1e-12);

			double discount = termStructure.Discount(0, false);

			// check deposits
			for( int i=0; i<depositData.GetLength(0); i++)
			{
				Period tenor = Period.Parse((string)depositData[i,0]);
				double rate = (double)depositData[i,1] / 100.0;
				// IIndex index = new Euribor("DepositData", tenor, termStructure);
				IXibor index = new Xibor("DepositData", 
					tenor, settlementDays, Currency.EUR,
					calendar, true, rollingConvention, dayCounter, termStructure);
				double estimatedRate = index.Fixing(todaysDate);
				Assert( String.Format( "Deposit rate for {0}: tolerance exceeded", tenor),
					Math.Abs(estimatedRate - rate) <= TOLERANCE);
			}

			// check swaps by repricing
			// IIndex index = new Euribor("DepositData", tenor, termStructure);
			IXibor swapIndex = new Xibor("SwapData", 
				12/floatingFrequency, TimeUnit.Months, 
				settlementDays, Currency.EUR,
				calendar, true, rollingConvention, dayCounter, termStructure);
			for( int i=0; i<swapData.GetLength(0); i++)
			{
				Period tenor = Period.Parse((string)swapData[i,0]);
				double rate = (double)swapData[i,1] / 100.0;
				SimpleSwap swap = new SimpleSwap(true, settlementDate, 
					tenor.Length, tenor.Units, calendar, swapRollingConvention,
					100.0, fixedFrequency, 0.0, fixedIsAdjusted, fixedDayCount,
					floatingFrequency, swapIndex, fixingDays, 0.0, termStructure);
				double estimatedRate = swap.FairRate;
				Assert( String.Format( "Swap rate for {0}: tolerance exceeded", tenor),
					Math.Abs(estimatedRate - rate) <= TOLERANCE);
			}

		}


		/// <summary>
		/// Testing Array.BinarySearch.
		/// </summary>
		public void TestBinarySearch()
		{
			double[] times = new double[] 
				{ 0.0, 0.3, 0.8, 0.15001, 1.0 };

			// sort and check resulting order
			Array.Sort(times);
			
			int i = Array.BinarySearch(times, 0.3);
			Assertion.Assert( "Exact match of value", i == 2);
			i = Array.BinarySearch(times, 0.15);
			Assertion.Assert( "Search for non-existing value", i == ~1);
			i = Array.BinarySearch(times, 1.2); 
			Assertion.Assert( "Search for larger than max value", 
				i == ~times.Length);

		}

	}

}

