// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestRateHelpers.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *  Revision 1.2  2002/08/30 18:23:25  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.1  2002/08/07 19:15:21  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.QuantLib.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Stochastix.Business;
	using Stochastix.Business.Calendars;
	using Stochastix.Business.DayCounters;
	using Stochastix.QuantLib.TermStructures;

	/// <summary>
	/// 
	/// </summary>
	public class TestRateHelpers: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestRateHelpers(String name): base(name) {}

		// declare private members here used for test setup

		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
			termStructure = new FlatForward(
				DateTime.Today, DateTime.Today.AddDays(2),
				0.04, Actual365.Instance);
		}

		private ITermStructure termStructure;

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
			termStructure = null;
		}

		/// <summary>
		/// Testing the IComparable implementation of RateHelper.
		/// </summary>
		public void TestIComparable()
		{

			ArrayList rateHelpers = new ArrayList(5);

			rateHelpers.Add( new DepositRateHelper(
				0.04, 3, TimeUnit.Months, 2, TARGET.Instance, 
				RollingConvention.ModifiedFollowing, Thirty360EU.Instance));

			rateHelpers.Add( new DepositRateHelper(
				0.03, 1, TimeUnit.Months, 2, TARGET.Instance, 
				RollingConvention.ModifiedFollowing, Thirty360EU.Instance));
					
			rateHelpers.Add( new FraRateHelper(
				0.04, 6, 12, 2, TARGET.Instance, 
				RollingConvention.ModifiedFollowing, Thirty360EU.Instance));

			// prepare the rate helpers
			foreach ( IRateHelper rateHelper in rateHelpers)
				rateHelper.TermStructure = termStructure;

			// sort and check resulting order
			rateHelpers.Sort(null);
			DateTime lastMaturity = DateTime.Today;
			foreach ( IRateHelper rateHelper in rateHelpers) 
			{
				DateTime maturity = rateHelper.Maturity();
				Assertion.Assert("RateHelper IComparable implementation",
					maturity >= lastMaturity);
					lastMaturity = maturity;
			}

		}

	}

}
