// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: RcsIdAttribute.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:03:25  jens
 *  Initial import
 *
 *  Revision 1.4  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.3  2002/10/15 12:46:15  stochastix
 *  updated license
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes

namespace Stochastix.Utilities
{
	/// <summary>
	/// Add RCS revision information as metdata.
	/// </summary>
	/// <remarks>
	/// On checkin, "$Id: RcsIdAttribute.cs,v 1.1.1.1 2003/02/01 19:03:25 jens Exp $" is replaced with the RCS ID of the file
	/// containing your class definition. Use this attribute to
	/// store RCS information within your assemblies.
	/// </remarks>
	/// <example>
	/// [C#]
	/// <code>
	/// [ RcsId("$Id: RcsIdAttribute.cs,v 1.1.1.1 2003/02/01 19:03:25 jens Exp $") ]
	/// public class FooBar {
	///		public string message;
	///	}
	/// </code>
	/// </example>
	[ AttributeUsage(
		  AttributeTargets.Assembly | AttributeTargets.Class | 
		  AttributeTargets.Enum | AttributeTargets.Interface | 
		  AttributeTargets.Module | AttributeTargets.Struct, 
		  Inherited = false
		  ) 
	]
	[ ComVisible(false) ]
	public sealed class RcsIdAttribute : Attribute
	{
		/// <summary>
		/// Attach the given RCS tag as an object attribute.
		/// </summary>
		/// <param name="tag">RCS tag, usually <c>"$Id: RcsIdAttribute.cs,v 1.1.1.1 2003/02/01 19:03:25 jens Exp $"</c>.</param>
		public RcsIdAttribute(string tag)
		{
			this.tag = tag;
		}

		private string tag;

		/// <summary>
		/// RCS tag attributed to a class.
		/// </summary>
		public string Tag
		{
			get{ return tag; }
		}
	}
}
