// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: RcsLogAttribute.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:03:25  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes

namespace Stochastix.Utilities
{
	/// <summary>
	/// Add RCS log data to your assemblies.
	/// </summary>
	/// <remarks>
	/// On checkin, "$Log: RcsLogAttribute.cs,v $
	/// On checkin, "Revision 1.1.1.1  2003/02/01 19:03:25  jens
	/// On checkin, "Initial import
	/// On checkin, "" is replaced with the RCS log of the file
	/// containing your class definition. Use this attribute to
	/// store RCS information within your assemblies.
	/// </remarks>
	/// <example>
	/// See <see cref="RcsIdAttribute"/> for an example.
	/// </example>
	[ AttributeUsage(
		  AttributeTargets.Assembly | AttributeTargets.Class | 
		  AttributeTargets.Enum | AttributeTargets.Interface | 
		  AttributeTargets.Module | AttributeTargets.Struct, 
		  Inherited = false
		  ) 
	]
	[ ComVisible(false) ]
	public sealed class RcsLogAttribute : Attribute
	{
		/// <summary>
		/// Attach the given RCS log as an object attribute.
		/// </summary>
		/// <param name="log">RCS log.</param>
		public RcsLogAttribute(string log)
		{
			this.log = log;
		}
	
		private string log;

		/// <summary>
		/// RCS log attributed to a class.
		/// </summary>
		public string Log
		{
			get{ return log; }
		}
	}
}
